#include <string.h>
#include "emu.h"
#include "video.h"
#include "bios.h"
#include "dbcs.h"

/*
 * Is the cell the second half of a double byte char?
 */
int is_dbcs_tail(int x, int y)
{
    int xx;
    us *sp = screen_adr + y * co;

    for (xx = 0; xx < x; xx++, sp++)
	if (is_dbcs_lead_byte(CHAR(sp))) {
	    xx++;
	    sp++;
	}
    return (xx > x);
}

/*
 * Japanese multi-byte Shift-JIS to wide JIS code conversion
 */
static unsigned short cp932_to_wchar(unsigned char c[2])
{
    unsigned byte1 = c[0];
    unsigned byte2 = c[1];

    byte1 = byte1 + byte1 - ((byte1 <= 0x9f) ? 0xe1 : 0x161);
    if (byte2 < 0x9f)
	byte2 = byte2 - ((byte2 > 0x7f) ? 0x20 : 0x1f);
    else {
	byte2 = byte2 - 0x7e;
	byte1++;
    }
    return (byte1 << 8) + byte2;
}

void change_dbcs_mode(const unsigned char *dbcs_vec, int new_cp)
{
    int c;

					/* dos data may be below 1000:0... */ 
    if (!dos_dbcs_vec && (int) dbcs_vec < 0x10000)
	dos_dbcs_vec = dbcs_vec;

    dbcs_codepage = new_cp;
    memset(dbcs_lead_map, 0, sizeof(dbcs_lead_map));
    if (!dbcs_vec || (!READ_BYTE(dbcs_vec) && !READ_BYTE(dbcs_vec+1))) {
	dbcs_mode = 0;
	goto setmode;
    }

    for ( ; READ_BYTE(dbcs_vec) || READ_BYTE(dbcs_vec+1); dbcs_vec += 2) {
	if (READ_BYTE(dbcs_vec) > READ_BYTE(dbcs_vec+1)) {
	    printf("malformed dbcs vec (start:%x > end:%x), ignoring...\n",
	    	READ_BYTE(dbcs_vec), READ_BYTE(dbcs_vec+1));
	    dbcs_mode = 0;
	    goto setmode;
	}
	for (c = READ_BYTE(dbcs_vec); c <= READ_BYTE(dbcs_vec+1); c++)
	    dbcs_lead_map[c] = 1;
    }

    switch (dbcs_codepage) {
    case 932:				/* Japanese Shift-JIS */
        dbcs_to_wchar = cp932_to_wchar;
	break;
    default:
        printf("unsupported DBCS code page %d!\n", dbcs_codepage);
	dbcs_mode = 0;
	goto setmode;
    }
        
    dbcs_mode = 1;

setmode:
    if (Video->setmode)
	Video->setmode(0, co, li);
}
